SystemOrganization addCategory: #'GT-EventRecorder-Server'!
SystemOrganization addCategory: 'GT-EventRecorder-Server-Core'!
SystemOrganization addCategory: 'GT-EventRecorder-Server-Tests'!

TestCase subclass: #GTERServerPutHandlerTest
	instanceVariableNames: 'receivedContents receiverSemaphore handler delegate server url data client location storage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Server-Tests'!

!GTERServerPutHandlerTest commentStamp: '<historical>' prior: 0!
A GTERServerDelegateTest is a test class for testing the behavior of GTERServerDelegate!

!GTERServerPutHandlerTest methodsFor: 'asserting' stamp: 'JurajKubelka 11/15/2015 13:21'!
assertReceiverSemaphore
	self deny: (receiverSemaphore waitTimeoutMSecs: 200)! !

!GTERServerPutHandlerTest methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 15:11'!
put: aByteArray 
	| key |
	key := (receivedContents size + 1) printString.
	receivedContents nextPut: key -> aByteArray.
	receiverSemaphore signal.
	^ key! !

!GTERServerPutHandlerTest methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 13:19'!
server
	^ server! !

!GTERServerPutHandlerTest methodsFor: 'running' stamp: 'JurajKubelka 11/15/2015 12:56'!
setUp
	super setUp.
	self setUpServer.
	client := ZnClient new
		url: url;
		followRedirects: false;
		numberOfRetries: 0;
		enforceHttpSuccess: true;
		logToTranscript;
		yourself.! !

!GTERServerPutHandlerTest methodsFor: 'running' stamp: 'JurajKubelka 11/15/2015 14:27'!
setUpServer
	| subscription |
	receivedContents := WaitfreeQueue new.
	receiverSemaphore := Semaphore new.
	handler := GTERServerPutHandler new.
	handler storage: self.
	delegate := ZnDispatcherDelegate new.
	delegate map: '/gt/events' to: handler.
	server := ZnServer on: 0.
	server delegate: delegate.
	server route: UUID new asString.
	subscription := ZnLogEvent announcer 
							when: ZnServerSocketBoundEvent 
							do: [ :announcement | announcement serverId = self server route 
										ifTrue: [ receiverSemaphore signal ] ]
							for: self.
	server start.
	self assertReceiverSemaphore.
	ZnLogEvent announcer removeSubscription: subscription.
	url := ZnUrl new
		scheme: #http;
		host: NetNameResolver loopBackName;
		port: server serverSocket port;
		addPathSegment: 'gt';
		addPathSegment: 'events';
		yourself.! !

!GTERServerPutHandlerTest methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 14:49'!
storage
	storage ifNil: [ storage := Dictionary new ].
	receivedContents flush: [ :association |
		storage at: association key put: association value ].
	^ storage! !

!GTERServerPutHandlerTest methodsFor: 'running' stamp: 'JurajKubelka 11/15/2015 12:51'!
tearDown
	super tearDown.
	server stop.! !

!GTERServerPutHandlerTest methodsFor: 'tests' stamp: 'JurajKubelka 11/15/2015 12:55'!
testPutByteArrayEntity
	data := #[1 2 3 4].
	client
		entity: (ZnByteArrayEntity bytes: data);
		put.
	self assert: client isSuccess.
	self assert: client isCreated.
	location := client response location asZnUrl.
	self assert: (self storage at: location lastPathSegment) equals: data.
! !

Object subclass: #GTERServerPutHandler
	instanceVariableNames: 'storage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Server-Core'!

!GTERServerPutHandler methodsFor: 'private' stamp: 'JurajKubelka 11/15/2015 15:11'!
saveByteArray: aByteArray
	"storage is responsible for storing aByteArray.
	It returns location."
	^ self storage put: aByteArray.! !

!GTERServerPutHandler methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 13:18'!
storage
	^ storage! !

!GTERServerPutHandler methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 13:18'!
storage: anObject
	storage := anObject! !

!GTERServerPutHandler methodsFor: 'evaluating' stamp: 'JurajKubelka 11/15/2015 14:55'!
value: aZnRequest value: aZnResponse
	| badRequest key |
	badRequest := [ ^ ZnResponse badRequest: aZnRequest ].
	(aZnRequest hasEntity and: [ aZnRequest contentType matches: ZnMimeType applicationOctetStream ])
		ifTrue: [ 
			[ key := self saveByteArray: aZnRequest entity bytes ] on: Error do: badRequest.
			^ ZnResponse created: key ].
	badRequest value.! !
